package com.userauth.restuserauth.security;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

@Service
public class JwtBlacklistService {

    private static final Logger logger = LoggerFactory.getLogger(JwtBlacklistService.class);

    // 使用 ConcurrentHashMap 作为内存中的黑名单存储。
    private final Map<String, Date> blacklist = new ConcurrentHashMap<>();

    /**
     * 将令牌及其过期时间加入黑名单。
     * @param token 要加入黑名单的 JWT。
     * @param expirationDate 令牌的过期时间。
     */
    public void blacklistToken(String token, Date expirationDate) {
        if (token != null && expirationDate != null) {
            blacklist.put(token, expirationDate);
        }
    }

    /**
     * 检查令牌是否存在于黑名单中。
     * @param token 要检查的 JWT。
     * @return 如果令牌在黑名单中，则返回 true，否则返回 false。
     */
    public boolean isBlacklisted(String token) {
        return blacklist.containsKey(token);
    }

    /**
     * 定时任务，定期清理黑名单中已过期的令牌。
     * cron 表达式 "0 0 * * * *" 表示每小时的 0 分 0 秒执行一次。
     */
    @Scheduled(cron = "*/30 * * * * *")
    public void cleanupExpiredTokens() {
        logger.info("开始清理黑名单中的过期令牌... 当前黑名单大小：{}", blacklist.size());
        
        // 使用 removeIf 安全地从 ConcurrentHashMap 中移除过期的条目
        blacklist.entrySet().removeIf(entry -> entry.getValue().before(new Date()));
        
        logger.info("清理完成。当前黑名单大小：{}", blacklist.size());
    }
}
